@@ -22,14 +22,13 @@
 from __future__ import absolute_import
 import android
 
-from electrum import SimpleConfig, Wallet, WalletStorage, format_satoshis, mnemonic_encode, mnemonic_decode
-from electrum.bitcoin import is_valid
+from electrum import SimpleConfig, Wallet, WalletStorage, format_satoshis
+from electrum.bitcoin import is_address
 from electrum import util
 from decimal import Decimal
 import datetime, re
 
 
-
 def modal_dialog(title, msg = None):
     droid.dialogCreateAlert(title,msg)
     droid.dialogSetPositiveButtonText('OK')
@@ -69,25 +68,15 @@ def modal_question(q, msg, pos_text = 'OK', neg_text = 'Cancel'):
     return result.get('which') == 'positive'
 
 def edit_label(addr):
-    v = modal_input('Edit label',None,wallet.labels.get(addr))
+    v = modal_input('Edit label', None, wallet.labels.get(addr))
     if v is not None:
-        if v:
-            wallet.labels[addr] = v
-        else:
-            if addr in wallet.labels.keys():
-                wallet.labels.pop(addr)
-        wallet.update_tx_history()
-        wallet.save()
+        wallet.set_label(addr, v)
         droid.fullSetProperty("labelTextView", "text", v)
 
 def select_from_contacts():
     title = 'Contacts:'
     droid.dialogCreateAlert(title)
-    l = []
-    for i in range(len(wallet.addressbook)):
-        addr = wallet.addressbook[i]
-        label = wallet.labels.get(addr,addr)
-        l.append( label )
+    l = contacts.keys()
     droid.dialogSetItems(l)
     droid.dialogSetPositiveButtonText('New contact')
     droid.dialogShow()
@@ -98,43 +87,20 @@ def select_from_contacts():
         return 'newcontact'
 
     result = response.get('item')
-    print result
     if result is not None:
-        addr = wallet.addressbook[result]
-        return addr
+        t, v = contacts.get(result)
+        return v
 
 
-def select_from_addresses():
-    droid.dialogCreateAlert("Addresses:")
-    l = []
-    addresses = wallet.addresses()
-    for i in range(len(addresses)):
-        addr = addresses[i]
-        label = wallet.labels.get(addr,addr)
-        l.append( label )
-    droid.dialogSetItems(l)
-    droid.dialogShow()
-    response = droid.dialogGetResponse()
-    result = response.result.get('item')
-    droid.dialogDismiss()
-    if result is not None:
-        addr = addresses[result]
-        return addr
-
 
 def protocol_name(p):
     if p == 't': return 'TCP'
-    if p == 'h': return 'HTTP'
     if p == 's': return 'SSL'
-    if p == 'g': return 'HTTPS'
 
 
 def protocol_dialog(host, protocol, z):
-    droid.dialogCreateAlert('Protocol',host)
-    if z:
-        protocols = z.keys()
-    else:
-        protocols = 'thsg'
+    droid.dialogCreateAlert('Protocol', host)
+    protocols = filter(lambda x: x in "ts", z.keys())
     l = []
     current = protocols.index(protocol)
     for p in protocols:
@@ -147,8 +113,10 @@ def protocol_dialog(host, protocol, z):
     selected_item = droid.dialogGetSelectedItems().result
     droid.dialogDismiss()
 
-    if not response: return
-    if not selected_item: return
+    if not response:
+        return
+    if not selected_item:
+        return
     if response.get('which') == 'positive':
         return protocols[selected_item[0]]
 
@@ -211,7 +179,8 @@ def make_layout(s, scrollable = False):
 
 
 def main_layout():
-    return make_layout("""
+    h = get_history_layout(15)
+    l = make_layout("""
         <TextView android:id="@+id/balanceTextView" 
                 android:layout_width="match_parent"
                 android:text=""
@@ -229,39 +198,55 @@ def main_layout():
                 android:textAppearance="?android:attr/textAppearanceLarge" 
                 android:gravity="center_vertical|center_horizontal|center">
         </TextView>
-
-        %s """%get_history_layout(15),True)
+        %s """%h,True)
+    return l
 
 
 
-def qr_layout(addr):
-    return make_layout("""
-
-     <TextView android:id="@+id/addrTextView" 
-                android:layout_width="match_parent"
-                android:layout_height="50" 
-                android:text="%s"
-                android:textAppearance="?android:attr/textAppearanceLarge" 
-                android:gravity="center_vertical|center_horizontal|center">
-     </TextView>
-
-     <ImageView
-        android:id="@+id/qrView"
-        android:gravity="center"
+def qr_layout(addr, amount, message):
+    addr_view= """
+     <TextView android:id="@+id/addrTextView"
+    android:layout_width="match_parent"
+    android:layout_height="50"
+    android:text="%s"
+    android:textAppearance="?android:attr/textAppearanceLarge"
+    android:gravity="center_vertical|center_horizontal|center">
+    </TextView>"""%addr
+    if amount:
+        amount_view = """
+        <TextView android:id="@+id/amountTextView"
         android:layout_width="match_parent"
-        android:layout_height="350"
-        android:antialias="false"
-        android:src="file:///sdcard/sl4a/qrcode.bmp" /> 
+        android:layout_height="50"
+        android:text="Amount: %s"
+        android:textAppearance="?android:attr/textAppearanceLarge"
+        android:gravity="center_vertical|center_horizontal|center">
+        </TextView>"""%format_satoshis(amount)
+    else:
+        amount_view = ""
+    if message:
+        message_view = """
+        <TextView android:id="@+id/messageTextView"
+        android:layout_width="match_parent"
+        android:layout_height="50"
+        android:text="Message: %s"
+        android:textAppearance="?android:attr/textAppearanceLarge"
+        android:gravity="center_vertical|center_horizontal|center">
+        </TextView>"""%message
+    else:
+        message_view = ""
 
-     <TextView android:id="@+id/labelTextView" 
-                android:layout_width="match_parent"
-                android:layout_height="50" 
-                android:text="%s"
-                android:textAppearance="?android:attr/textAppearanceLarge" 
-                android:gravity="center_vertical|center_horizontal|center">
-     </TextView>
-
-     """%(addr,wallet.labels.get(addr,'')), True)
+    return make_layout("""
+    %s
+    %s
+    %s
+    <ImageView
+    android:id="@+id/qrView"
+    android:gravity="center"
+    android:layout_width="match_parent"
+    android:layout_height="350"
+    android:antialias="false"
+    android:src="file:///sdcard/sl4a/qrcode.bmp" />
+    """%(addr_view, amount_view, message_view), True)
 
 payto_layout = make_layout("""
 
@@ -293,12 +278,12 @@ payto_layout = make_layout("""
         <TextView android:id="@+id/labelTextView" 
                 android:layout_width="match_parent"
                 android:layout_height="wrap_content" 
-                android:text="Description:"
+                android:text="Message:"
                 android:textAppearance="?android:attr/textAppearanceLarge" 
                 android:gravity="left">
         </TextView>
 
-        <EditText android:id="@+id/label"
+        <EditText android:id="@+id/message"
                 android:layout_width="match_parent"
                 android:layout_height="wrap_content" 
                 android:tag="Tag Me" android:inputType="text">
@@ -332,13 +317,12 @@ settings_layout = make_layout(""" <ListView
            android:layout_height="wrap_content" />""")
 
 
-
 def get_history_values(n):
     values = []
-    h = wallet.get_tx_history()
+    h = wallet.get_history()
     length = min(n, len(h))
     for i in range(length):
-        tx_hash, conf, is_mine, value, fee, balance, timestamp = h[-i-1]
+        tx_hash, conf, value, timestamp, balance = h[-i-1]
         try:
             dt = datetime.datetime.fromtimestamp( timestamp )
             if dt.date() == dt.today().date():
@@ -347,10 +331,10 @@ def get_history_values(n):
                 time_str = str( dt.date() )
         except Exception:
             time_str = 'pending'
-
         conf_str = 'v' if conf else 'o'
         label, is_default_label = wallet.get_label(tx_hash)
-        values.append((conf_str, '  ' + time_str, '  ' + format_satoshis(value,True), '  ' + label ))
+        label = label.replace('<','').replace('>','')
+        values.append((conf_str, '  ' + time_str, '  ' + format_satoshis(value, True), '  ' + label))
 
     return values
 
@@ -445,7 +429,7 @@ def update_layout():
 
 
 
-def pay_to(recipient, amount, fee, label):
+def pay_to(recipient, amount, label):
 
     if wallet.use_encryption:
         password  = droid.dialogGetPassword('Password').result
@@ -457,14 +441,14 @@ def pay_to(recipient, amount, fee, label):
     droid.dialogShow()
 
     try:
-        tx = wallet.mktx( [(recipient, amount)], password, fee)
+        tx = wallet.mktx([('address', recipient, amount)], password)
     except Exception as e:
         modal_dialog('error', e.message)
         droid.dialogDismiss()
         return
 
-    if label: 
-        wallet.labels[tx.hash()] = label
+    if label:
+        wallet.set_label(tx.hash(), label)
 
     droid.dialogDismiss()
 
@@ -485,17 +469,18 @@ def make_new_contact():
     code = droid.scanBarcode()
     r = code.result
     if r:
-        data = r['extras']['SCAN_RESULT']
+        data = str(r['extras']['SCAN_RESULT']).strip()
         if data:
             if re.match('^bitcoin:', data):
-                address, _, _, _, _, _, _ = util.parse_url(data)
-            elif is_valid(data):
+                address, _, _, _, _ = util.parse_URI(data)
+            elif is_address(data):
                 address = data
             else:
                 address = None
             if address:
                 if modal_question('Add to contacts?', address):
-                    wallet.add_contact(address)
+                    # fixme: ask for key
+                    contacts[address] = ('address', address)
         else:
             modal_dialog('Invalid address', data)
 
@@ -557,16 +542,14 @@ def main_loop():
 
             elif out == "receive":
                 global receive_addr
-                receive_addr = select_from_addresses()
-                if receive_addr:
-                    amount = modal_input('Amount', 'Amount you want receive. ', '', "numberDecimal")
-                    if amount:
-                        receive_addr = 'bitcoin:%s?amount=%s'%(receive_addr, amount)
-
-                if not receive_addr:
+                domain = wallet.addresses(include_change = False)
+                for addr in domain:
+                    if not wallet.history.get(addr):
+                        receive_addr = addr
+                        break
+                else:
                     out = None
 
-
     return out
                     
 
@@ -591,10 +574,10 @@ def payto_loop():
 
                 droid.fullQuery()
                 recipient = droid.fullQueryDetail("recipient").result.get('text')
-                label  = droid.fullQueryDetail("label").result.get('text')
+                message = droid.fullQueryDetail("message").result.get('text')
                 amount = droid.fullQueryDetail('amount').result.get('text')
 
-                if not is_valid(recipient):
+                if not is_address(recipient):
                     modal_dialog('Error','Invalid Bitcoin address')
                     continue
 
@@ -604,27 +587,32 @@ def payto_loop():
                     modal_dialog('Error','Invalid amount')
                     continue
 
-                result = pay_to(recipient, amount, wallet.fee, label)
+                result = pay_to(recipient, amount, message)
                 if result:
                     out = 'main'
 
             elif id=="buttonContacts":
                 addr = select_from_contacts()
-                droid.fullSetProperty("recipient","text",addr)
+                droid.fullSetProperty("recipient", "text", addr)
 
             elif id=="buttonQR":
                 code = droid.scanBarcode()
                 r = code.result
                 if r:
-                    data = r['extras']['SCAN_RESULT']
+                    data = str(r['extras']['SCAN_RESULT']).strip()
                     if data:
+                        print "data", data
                         if re.match('^bitcoin:', data):
-                            payto, amount, label, _, _, _, _ = util.parse_url(data)
-                            droid.fullSetProperty("recipient", "text",payto)
+                            payto, amount, label, message, _ = util.parse_URI(data)
+                            if amount:
+                                amount = str(amount/100000000)
+                            droid.fullSetProperty("recipient", "text", payto)
                             droid.fullSetProperty("amount", "text", amount)
-                            droid.fullSetProperty("label", "text", label)
-                        else:
+                            droid.fullSetProperty("message", "text", message)
+                        elif is_address(data):
                             droid.fullSetProperty("recipient", "text", data)
+                        else:
+                            modal_dialog('Error','cannot parse QR code\n'+data)
 
                     
         elif event["name"] in menu_commands:
@@ -642,24 +630,43 @@ def payto_loop():
 
 
 receive_addr = ''
+receive_amount = None
+receive_message = None
+
 contact_addr = ''
 recipient = ''
 
 def receive_loop():
+    global receive_addr, receive_amount, receive_message
+    print "receive loop"
+    receive_URI = util.create_URI(receive_addr, receive_amount, receive_message)
+    make_bitmap(receive_URI)
+    droid.fullShow(qr_layout(receive_addr, receive_amount, receive_message))
     out = None
     while out is None:
         event = droid.eventWait().result
-        print "got event", event
-        if event["name"]=="key":
+        if not event:
+            continue
+
+        elif event["name"]=="key":
             if event["data"]["key"] == '4':
                 out = 'main'
 
         elif event["name"]=="clipboard":
-            droid.setClipboard(receive_addr)
-            modal_dialog('Address copied to clipboard',receive_addr)
+            droid.setClipboard(receive_URI)
+            modal_dialog('URI copied to clipboard', receive_URI)
 
-        elif event["name"]=="edit":
-            edit_label(receive_addr)
+        elif event["name"]=="amount":
+            amount = modal_input('Amount', 'Amount you want to receive (in BTC). ', format_satoshis(receive_amount) if receive_amount else None, "numberDecimal")
+            if amount is not None:
+                receive_amount = int(100000000 * Decimal(amount)) if amount else None
+                out = 'receive'
+
+        elif event["name"]=="message":
+            message = modal_input('Message', 'Message in your request', receive_message)
+            if message is not None:
+                receive_message = unicode(message)
+                out = 'receive'
 
     return out
 
@@ -717,13 +724,12 @@ def show_seed():
         password = None
     
     try:
-        seed = wallet.get_seed(password)
+        seed = wallet.get_mnemonic(password)
     except Exception:
         modal_dialog('error','incorrect password')
         return
 
-    modal_dialog('Your seed is',seed)
-    modal_dialog('Mnemonic code:', ' '.join(mnemonic_encode(seed)) )
+    modal_dialog('Your seed is', seed)
 
 def change_password_dialog():
     if wallet.use_encryption:
@@ -733,7 +739,7 @@ def change_password_dialog():
         password = None
 
     try:
-        wallet.get_seed(password)
+        wallet.check_password(password)
     except Exception:
         modal_dialog('error','incorrect password')
         return
@@ -760,12 +766,12 @@ def settings_loop():
 
 
     def set_listview():
-        host, port, p = network.default_server.split(':')
-        fee = str( Decimal( wallet.fee)/100000000 )
+        host, port, p, proxy_config, auto_connect = network.get_parameters()
+        fee = str( Decimal( wallet.fee_per_kb)/100000000 )
         is_encrypted = 'yes' if wallet.use_encryption else 'no'
         protocol = protocol_name(p)
         droid.fullShow(settings_layout)
-        droid.fullSetList("myListView",['Server: ' + host, 'Protocol: '+ protocol, 'Port: '+port, 'Transaction fee: '+fee, 'Password: '+is_encrypted, 'Seed'])
+        droid.fullSetList("myListView",['Server: ' + host, 'Protocol: '+ protocol, 'Port: '+port, 'Transaction fee/kb: '+fee, 'Password: '+is_encrypted, 'Seed'])
 
     set_listview()
 
@@ -783,7 +789,7 @@ def settings_loop():
 
         if name == "itemclick":
             pos = event["data"]["position"]
-            host, port, protocol = network.default_server.split(':')
+            host, port, protocol, proxy_config, auto_connect = network.get_parameters()
             network_changed = False
 
             if pos == "0": #server
@@ -796,9 +802,10 @@ def settings_loop():
             elif pos == "1": #protocol
                 if host in servers:
                     protocol = protocol_dialog(host, protocol, servers[host])
-                    z = servers[host]
-                    port = z[protocol]
-                    network_changed = True
+                    if protocol:
+                        z = servers[host]
+                        port = z[protocol]
+                        network_changed = True
 
             elif pos == "2": #port
                 a_port = modal_input('Port number', 'If you use a public server, this field is set automatically when you set the protocol', port, "number")
@@ -807,7 +814,8 @@ def settings_loop():
                     network_changed = True
 
             elif pos == "3": #fee
-                fee = modal_input('Transaction fee', 'The fee will be this amount multiplied by the number of inputs in your transaction. ', str( Decimal( wallet.fee)/100000000 ), "numberDecimal")
+                fee = modal_input('Transaction fee', 'The fee will be this amount multiplied by the number of inputs in your transaction. ',
+                                  str(Decimal(wallet.fee_per_kb)/100000000 ), "numberDecimal")
                 if fee:
                     try:
                         fee = int( 100000000 * Decimal(fee) )
@@ -853,7 +861,8 @@ def add_menu(s):
         droid.addOptionsMenuItem("Settings","settings",None,"")
     elif s == 'receive':
         droid.addOptionsMenuItem("Copy","clipboard",None,"")
-        droid.addOptionsMenuItem("Label","edit",None,"")
+        droid.addOptionsMenuItem("Amount","amount",None,"")
+        droid.addOptionsMenuItem("Message","message",None,"")
     elif s == 'contacts':
         droid.addOptionsMenuItem("Copy","clipboard",None,"")
         droid.addOptionsMenuItem("Label","edit",None,"")
@@ -861,17 +870,15 @@ def add_menu(s):
         #droid.addOptionsMenuItem("Delete","deletecontact",None,"")
 
 
-def make_bitmap(addr):
+def make_bitmap(data):
     # fixme: this is highly inefficient
     droid.dialogCreateSpinnerProgress("please wait")
     droid.dialogShow()
     try:
-        import pyqrnative, bmp
-        qr = pyqrnative.QRCode(4, pyqrnative.QRErrorCorrectLevel.L)
-        qr.addData(addr)
-        qr.make()
-        k = qr.getModuleCount()
-        assert k == 33
+        import qrcode
+        from electrum import bmp
+        qr = qrcode.QRCode()
+        qr.add_data(data)
         bmp.save_qrcode(qr,"/sdcard/sl4a/qrcode.bmp")
     finally:
         droid.dialogDismiss()
@@ -883,45 +890,63 @@ droid = android.Android()
 menu_commands = ["send", "receive", "settings", "contacts", "main"]
 wallet = None
 network = None
+contacts = None
 
 class ElectrumGui:
 
     def __init__(self, config, _network):
-        global wallet, network
+        global wallet, network, contacts
         network = _network
         network.register_callback('updated', update_callback)
         network.register_callback('connected', update_callback)
         network.register_callback('disconnected', update_callback)
         network.register_callback('disconnecting', update_callback)
         
+        contacts = util.StoreDict(config, 'contacts')
+
         storage = WalletStorage(config)
         if not storage.file_exists:
             action = self.restore_or_create()
-            if not action: exit()
+            if not action:
+                exit()
+
+            password  = droid.dialogGetPassword('Choose a password').result
+            if password:
+                password2  = droid.dialogGetPassword('Confirm password').result
+                if password != password2:
+                    modal_dialog('Error','passwords do not match')
+                    exit()
+            else:
+                # set to None if it's an empty string
+                password = None
 
-            wallet = Wallet(storage)
             if action == 'create':
-                wallet.init_seed(None)
-                self.show_seed()
-                wallet.save_seed()
-                wallet.synchronize()  # generate first addresses offline
-                
+                wallet = Wallet(storage)
+                seed = wallet.make_seed()
+                modal_dialog('Your seed is:', seed)
+                wallet.add_seed(seed, password)
+                wallet.create_master_keys(password)
+                wallet.create_main_account(password)
             elif action == 'restore':
                 seed = self.seed_dialog()
                 if not seed:
                     exit()
-                wallet.init_seed(str(seed))
-                wallet.save_seed()
+                if not Wallet.is_seed(seed):
+                    exit()
+                wallet = Wallet.from_seed(seed, password, storage)
             else:
                 exit()
 
+            msg = "Creating wallet" if action == 'create' else "Restoring wallet"
+            droid.dialogCreateSpinnerProgress("Electrum", msg)
+            droid.dialogShow()
             wallet.start_threads(network)
-
             if action == 'restore':
-                if not self.restore_wallet():
-                    exit()
-
-            self.password_dialog()
+                wallet.restore(lambda x: None)
+            else:
+                wallet.synchronize()
+            droid.dialogDismiss()
+            droid.vibrate()
 
         else:
             wallet = Wallet(storage)
@@ -941,13 +966,11 @@ class ElectrumGui:
                 s = payto_loop()
 
             elif s == 'receive':
-                make_bitmap(receive_addr)
-                droid.fullShow(qr_layout(receive_addr))
                 s = receive_loop()
 
             elif s == 'contacts':
                 make_bitmap(contact_addr)
-                droid.fullShow(qr_layout(contact_addr))
+                droid.fullShow(qr_layout(contact_addr, None, None))
                 s = contacts_loop()
 
             elif s == 'settings':
@@ -970,12 +993,11 @@ class ElectrumGui:
         if not response: return
         if response.get('which') == 'negative':
             return
-
         return 'restore' if response.get('which') == 'neutral' else 'create'
 
 
     def seed_dialog(self):
-        if modal_question("Enter your seed","Input method",'QR Code', 'mnemonic'):
+        if modal_question("Enter your seed", "Input method", 'QR Code', 'mnemonic'):
             code = droid.scanBarcode()
             r = code.result
             if r:
@@ -983,49 +1005,7 @@ class ElectrumGui:
             else:
                 return
         else:
-            m = modal_input('Mnemonic','please enter your code')
-            try:
-                seed = mnemonic_decode(m.split(' '))
-            except Exception:
-                modal_dialog('error: could not decode this seed')
-                return
-
+            seed = modal_input('Mnemonic', 'please enter your code')
         return str(seed)
 
 
-    def network_dialog(self):
-        return True
-
-    def verify_seed(self):
-        wallet.save_seed()
-        return True
-        
-    def show_seed(self):
-        modal_dialog('Your seed is:', wallet.seed)
-        modal_dialog('Mnemonic code:', ' '.join(mnemonic_encode(wallet.seed)) )
-
-
-    def password_dialog(self):
-        change_password_dialog()
-
-
-    def restore_wallet(self):
-
-        msg = "recovering wallet..."
-        droid.dialogCreateSpinnerProgress("Electrum", msg)
-        droid.dialogShow()
-
-        wallet.restore(lambda x: None)
-
-        droid.dialogDismiss()
-        droid.vibrate()
-
-        if wallet.is_found():
-            wallet.fill_addressbook()
-            modal_dialog("recovery successful")
-        else:
-            if not modal_question("no transactions found for this seed","do you want to keep this wallet?"):
-                return False
-
-        return True
-
