@@ -6,8 +6,11 @@ import sys
 import random
 import traceback
 
+import socks
+import socket
+import json
 
-from util import user_dir, print_error, print_msg
+import util
 from bitcoin import *
 import interface
 from blockchain import Blockchain
@@ -31,7 +34,8 @@ DEFAULT_SERVERS = {
     'us.electrum.be':DEFAULT_PORTS,
 }
 
-DISCONNECTED_RETRY_INTERVAL = 60
+NODES_RETRY_INTERVAL = 60
+SERVER_RETRY_INTERVAL = 10
 
 
 def parse_servers(result):
@@ -71,7 +75,8 @@ def filter_protocol(servers, p):
     l = []
     for k, protocols in servers.items():
         if p in protocols:
-            l.append( ':'.join([k, protocols[p], p]) )
+            s = serialize_server(k, protocols[p], p)
+            l.append(s)
     return l
 
 
@@ -80,46 +85,75 @@ def pick_random_server(p='s'):
 
 from simple_config import SimpleConfig
 
+proxy_modes = ['socks4', 'socks5', 'http']
+
+def serialize_proxy(p):
+    if type(p) != dict:
+        return None
+    return ':'.join([p.get('mode'),p.get('host'), p.get('port')])
+
+def deserialize_proxy(s):
+    if s is None:
+        return None
+    if s.lower() == 'none':
+        return None
+    proxy = { "mode":"socks5", "host":"localhost" }
+    args = s.split(':')
+    n = 0
+    if proxy_modes.count(args[n]) == 1:
+        proxy["mode"] = args[n]
+        n += 1
+    if len(args) > n:
+        proxy["host"] = args[n]
+        n += 1
+    if len(args) > n:
+        proxy["port"] = args[n]
+    else:
+        proxy["port"] = "8080" if proxy["mode"] == "http" else "1080"
+    return proxy
+
+def deserialize_server(server_str):
+    host, port, protocol = str(server_str).split(':')
+    assert protocol in 'st'
+    int(port)
+    return host, port, protocol
+
+def serialize_server(host, port, protocol):
+    return str(':'.join([host, port, protocol]))
 
 
-class Network(threading.Thread):
+class Network(util.DaemonThread):
 
     def __init__(self, config=None):
         if config is None:
             config = {}  # Do not use mutables as default values!
-        threading.Thread.__init__(self)
-        self.daemon = True
+        util.DaemonThread.__init__(self)
         self.config = SimpleConfig(config) if type(config) == type({}) else config
         self.lock = threading.Lock()
         self.num_server = 8 if not self.config.get('oneserver') else 0
         self.blockchain = Blockchain(self.config, self)
         self.interfaces = {}
         self.queue = Queue.Queue()
-        self.running = False
         # Server for addresses and transactions
         self.default_server = self.config.get('server')
         # Sanitize default server
         try:
-            host, port, protocol = self.default_server.split(':')
-            assert protocol in 'st'
-            int(port)
+            deserialize_server(self.default_server)
         except:
             self.default_server = None
         if not self.default_server:
             self.default_server = pick_random_server('s')
 
-        self.protocol = self.default_server.split(':')[2]
+        self.protocol = deserialize_server(self.default_server)[2]
         self.irc_servers = {} # returned by interface (list from irc)
 
         self.disconnected_servers = set([])
-        self.disconnected_time = time.time()
 
-        self.recent_servers = self.config.get('recent_servers',[]) # successful connections
+        self.recent_servers = self.read_recent_servers()
         self.pending_servers = set()
 
         self.banner = ''
         self.interface = None
-        self.proxy = interface.deserialize_proxy(self.config.get('proxy'))
         self.heights = {}
         self.merkle_roots = {}
         self.utxo_roots = {}
@@ -132,7 +166,29 @@ class Network(threading.Thread):
         self.addresses = {}
         self.connection_status = 'connecting'
         self.requests_queue = Queue.Queue()
+        self.set_proxy(deserialize_proxy(self.config.get('proxy')))
 
+    def read_recent_servers(self):
+        if not self.config.path:
+            return []
+        path = os.path.join(self.config.path, "recent_servers")
+        try:
+            with open(path, "r") as f:
+                data = f.read()
+                return json.loads(data)
+        except:
+            return []
+
+    def save_recent_servers(self):
+        if not self.config.path:
+            return
+        path = os.path.join(self.config.path, "recent_servers")
+        s = json.dumps(self.recent_servers, indent=4, sort_keys=True)
+        try:
+            with open(path, "w") as f:
+                f.write(s)
+        except:
+            pass
 
     def get_server_height(self):
         return self.heights.get(self.default_server, 0)
@@ -140,7 +196,7 @@ class Network(threading.Thread):
     def server_is_lagging(self):
         h = self.get_server_height()
         if not h:
-            print_error('no height for main interface')
+            self.print_error('no height for main interface')
             return False
         lag = self.get_local_height() - self.get_server_height()
         return lag > 1
@@ -191,7 +247,7 @@ class Network(threading.Thread):
         return server
 
     def get_parameters(self):
-        host, port, protocol = self.default_server.split(':')
+        host, port, protocol = deserialize_server(self.default_server)
         auto_connect = self.config.get('auto_cycle', True)
         return host, port, protocol, self.proxy, auto_connect
 
@@ -204,7 +260,10 @@ class Network(threading.Thread):
         else:
             out = DEFAULT_SERVERS
             for s in self.recent_servers:
-                host, port, protocol = s.split(':')
+                try:
+                    host, port, protocol = deserialize_server(s)
+                except:
+                    continue
                 if host not in out:
                     out[host] = { protocol:port }
         return out
@@ -232,26 +291,32 @@ class Network(threading.Thread):
         self.response_queue = response_queue
         self.start_interfaces()
         t = threading.Thread(target=self.process_requests_thread)
-        t.daemon = True
         t.start()
         self.blockchain.start()
-        threading.Thread.start(self)
+        util.DaemonThread.start(self)
+
+    def set_proxy(self, proxy):
+        self.proxy = proxy
+        if proxy:
+            proxy_mode = proxy_modes.index(proxy["mode"]) + 1
+            socks.setdefaultproxy(proxy_mode, proxy["host"], int(proxy["port"]))
+            socket.socket = socks.socksocket
+            # prevent dns leaks, see http://stackoverflow.com/questions/13184205/dns-over-proxy
+            socket.getaddrinfo = lambda *args: [(socket.AF_INET, socket.SOCK_STREAM, 6, '', (args[0], args[1]))]
+        else:
+            socket.socket = socket._socketobject
+            socket.getaddrinfo = socket._socket.getaddrinfo
+
 
     def set_parameters(self, host, port, protocol, proxy, auto_connect):
-        proxy_str = interface.serialize_proxy(proxy)
-        server_str = ':'.join([ host, port, protocol ])
-        self.config.set_key('auto_cycle', auto_connect, True)
-        self.config.set_key("proxy", proxy_str, True)
-        self.config.set_key("server", server_str, True)
-        # abort if changes were not allowed by config
-        if self.config.get('server') != server_str or self.config.get('proxy') != proxy:
-            return
-
         if self.proxy != proxy or self.protocol != protocol:
-            print_error('restarting network')
-            self.proxy = proxy
+            self.print_error('restarting network')
+            for i in self.interfaces.values():
+                i.stop()
+                self.interfaces.pop(i.server)
+            self.set_proxy(proxy)
             self.protocol = protocol
-            for i in self.interfaces.values(): i.stop()
+            self.disconnected_servers = set([])
             if auto_connect:
                 #self.interface = None
                 return
@@ -263,6 +328,7 @@ class Network(threading.Thread):
                 if self.server_is_lagging():
                     self.stop_interface()
         else:
+            server_str = serialize_server(host, port, protocol)
             self.set_server(server_str)
 
 
@@ -277,9 +343,8 @@ class Network(threading.Thread):
 
     def switch_to_interface(self, interface):
         server = interface.server
-        print_error("switching to", server)
+        self.print_error("switching to", server)
         self.interface = interface
-        self.config.set_key('server', server, False)
         self.default_server = server
         self.send_subscriptions()
         self.set_status('connected')
@@ -294,7 +359,7 @@ class Network(threading.Thread):
         if self.default_server == server and self.interface.is_connected:
             return
 
-        if self.protocol != server.split(':')[2]:
+        if self.protocol != deserialize_server(server)[2]:
             return
 
         # stop the interface in order to terminate subscriptions
@@ -305,7 +370,6 @@ class Network(threading.Thread):
         self.set_status('connecting')
         # start interface
         self.default_server = server
-        self.config.set_key("server", server, True)
 
         if server in self.interfaces.keys():
             self.switch_to_interface( self.interfaces[server] )
@@ -320,8 +384,7 @@ class Network(threading.Thread):
             self.recent_servers.remove(s)
         self.recent_servers.insert(0,s)
         self.recent_servers = self.recent_servers[0:20]
-        self.config.set_key('recent_servers', self.recent_servers)
-
+        self.save_recent_servers()
 
     def add_interface(self, i):
         self.interfaces[i.server] = i
@@ -334,7 +397,7 @@ class Network(threading.Thread):
     def new_blockchain_height(self, blockchain_height, i):
         if self.is_connected():
             if self.server_is_lagging():
-                print_error( "Server is lagging", blockchain_height, self.get_server_height())
+                self.print_error("Server is lagging", blockchain_height, self.get_server_height())
                 if self.config.get('auto_cycle'):
                     self.set_server(i.server)
         self.notify('updated')
@@ -377,7 +440,7 @@ class Network(threading.Thread):
             except BaseException as e:
                 out['error'] = str(e)
                 traceback.print_exc(file=sys.stdout)
-                print_error("network error", str(e))
+                self.print_error("network error", str(e))
 
             self.response_queue.put(out)
             return
@@ -388,29 +451,45 @@ class Network(threading.Thread):
                 self.response_queue.put({'id':_id, 'result':self.addresses[addr]})
                 return
 
-        self.interface.send_request(request)
-
+        try:
+            self.interface.send_request(request)
+        except:
+            # put it back in the queue
+            self.print_error("warning: interface not ready for", request)
+            self.requests_queue.put(request)
+            time.sleep(0.1)
 
     def run(self):
+        server_retry_time = time.time()
+        nodes_retry_time = time.time()
         while self.is_running():
             try:
                 i, response = self.queue.get(timeout=0.1)
             except Queue.Empty:
-
+                now = time.time()
                 if len(self.interfaces) + len(self.pending_servers) < self.num_server:
                     self.start_random_interface()
                 if not self.interfaces:
-                    if time.time() - self.disconnected_time > DISCONNECTED_RETRY_INTERVAL:
-                        print_error('network: retrying connections')
+                    if now - nodes_retry_time > NODES_RETRY_INTERVAL:
+                        self.print_error('network: retrying connections')
                         self.disconnected_servers = set([])
-                        self.disconnected_time = time.time()
-
+                        nodes_retry_time = now
                 if not self.interface.is_connected:
-                    if time.time() - self.disconnected_time > DISCONNECTED_RETRY_INTERVAL:
-                        print_error("forcing reconnection")
-                        self.queue.put((self.interface, None))
-                        self.disconnected_time = time.time()
-
+                    if self.config.get('auto_cycle'):
+                        if self.interfaces:
+                            self.switch_to_random_interface()
+                    else:
+                        if self.default_server in self.interfaces.keys():
+                            self.switch_to_interface(self.interfaces[self.default_server])
+                        else:
+                            if self.default_server in self.disconnected_servers:
+                                if now - server_retry_time > SERVER_RETRY_INTERVAL:
+                                    self.disconnected_servers.remove(self.default_server)
+                                    server_retry_time = now
+                            else:
+                                if self.default_server not in self.pending_servers:
+                                    self.print_error("forcing reconnection")
+                                    self.interface = self.start_interface(self.default_server)
                 continue
 
             if response is not None:
@@ -426,34 +505,24 @@ class Network(threading.Thread):
                 self.add_recent_server(i)
                 i.send_request({'method':'blockchain.headers.subscribe','params':[]})
                 if i == self.interface:
-                    print_error('sending subscriptions to', self.interface.server)
+                    self.print_error('sending subscriptions to', self.interface.server)
                     self.send_subscriptions()
                     self.set_status('connected')
             else:
-                self.disconnected_servers.add(i.server)
                 if i.server in self.interfaces:
                     self.remove_interface(i)
                 if i.server in self.heights:
                     self.heights.pop(i.server)
                 if i == self.interface:
                     self.set_status('disconnected')
+                self.disconnected_servers.add(i.server)
 
-            if not self.interface.is_connected:
-                if self.config.get('auto_cycle'):
-                    self.switch_to_random_interface()
-                else:
-                    if self.default_server not in self.disconnected_servers:
-                        print_error("restarting main interface")
-                        if self.default_server in self.interfaces.keys():
-                            self.switch_to_interface(self.interfaces[self.default_server])
-                        else:
-                            self.interface = self.start_interface(self.default_server)
-
-
-        print_error("Network: Stopping interfaces")
+        self.print_error("stopping interfaces")
         for i in self.interfaces.values():
             i.stop()
 
+        self.print_error("stopped")
+
 
     def on_header(self, i, r):
         result = r.get('result')
@@ -470,7 +539,7 @@ class Network(threading.Thread):
 
         if i == self.interface:
             if self.server_is_lagging() and self.config.get('auto_cycle'):
-                print_error( "Server lagging, stopping interface")
+                self.print_error("Server lagging, stopping interface")
                 self.stop_interface()
             self.notify('updated')
 
@@ -489,15 +558,6 @@ class Network(threading.Thread):
         self.addresses[addr] = result
         self.response_queue.put(r)
 
-    def stop(self):
-        print_error("stopping network")
-        with self.lock:
-            self.running = False
-
-    def is_running(self):
-        with self.lock:
-            return self.running
-
     def get_header(self, tx_height):
         return self.blockchain.read_header(tx_height)
 
